//-----------------------------------------------------------------------------
// Copyright (C) 2008  PROCEQ SA, CH-8603 Schwerzenbach
//-----------------------------------------------------------------------------
// Package                : PqRemote
// Environment            : WIN32, MFC
// Tested on              : - Studio 2008 (V9)
//-----------------------------------------------------------------------------
// Source Code Management : $Workfile: PqRemote.h $
//                          $Revision: 16 $ $Date: 22.01.15 17:26 $ $NoKeywords: $
//-----------------------------------------------------------------------------
// Description:
//   This is the main header file of the PqRemote project. It declares
//   the methods exported in PqRemote.dll resp. PqRemote.lib
//
//   If you are programming C or C++ you should be able to include this header 
//   in your project.
//   If you are programming in another language, this is only
//   for documentation. 
//
//   If you created a interface header in your programming language for PqRemote
//   and are willing to share it, or if you have any suggestions, bugs found
//   etc. feel free to contact us at:                      software@proceq.com
//
//-----------------------------------------------------------------------------
// History (date author comment)
// -------
// 23-Oct-2008  MAP Initial
// 22-Jan-2015  MAP stdcall calling convention now for all projects.
//                  --> same dll works for Excel, too.
//-----------------------------------------------------------------------------

#pragma once

//-----------------------------------------------------------------------------
// The following ifdef block is the standard way of creating macros which make  
// exporting from a DLL simpler. All files within this DLL are compiled with 
// the PQREMOTE_EXPORTS symbol defined on the command line. this symbol should 
// not be defined on any project that uses this DLL. This way any other project 
// whose source files include this file see PQREMOTE_API functions as being 
// imported from a DLL, whereas this DLL sees symbols defined with this macro
// as being exported.
// define PQREMOTE_STATIC_LIB in your project if you use PqRemote.lib instead
// of the DLL
#ifdef PQREMOTE_EXPORTS
#define PQREMOTE_API __declspec(dllexport)
#else // not exports
#define PQREMOTE_API __declspec(dllimport)
#endif

#define CALLING_CONVENTION __stdcall



#ifdef __cplusplus
  extern "C" {  
#endif

//-----------------------------------------------------------------------------
// This macros declare which ProductVersion of the PqRemote.dll (PqRemote.lib) 
// is declared in this interface header file.
// E.g. ProductVersion = a.b.c.d  --> a is main version, b is side version.
// See PQ_CheckCompatibility()
#define PQ_REMOTE_INTERFACE_MAIN_VERSION 1
#define PQ_REMOTE_INTERFACE_SIDE_VERSION 0


//=== Exported methods ========================================================

//--- PQ_CheckCompatibility ---------------------------------------------------
// Description:
//   Checks if the DLL is compatible with this interface. 
// Parameter:
//   mainVersion [IN]: the PQ_REMOTE_INTERFACE_MAIN_VERSION of this header
//   sideVersion [IN]: the PQ_REMOTE_INTERFACE_SIDE_VERSION of this header
// Return value:
//   0= ok, version matches.
//   1= dll side is newer (dll is backwards compatible) --> ok
//   2= dll side is older (you should get a new dll)
//   3= dll main version does not match!
//   4= Software problem
PQREMOTE_API unsigned long CALLING_CONVENTION PQ_CheckCompatibility(const unsigned short mainVersion, 
                                                 const unsigned short sideVersion); 


//--- PQ_ConnectDevice --------------------------------------------------------
// Description:
//   Connect to a Proceq device (devType) using communication Port (commPort).
//   You get a device handle or 0 in case of failure.
//   Note: The parameters not needed for the choosen commPort are ignored.
//         E.g. if you choose 4(Ethernet), scanPort and portNr are ignored.
// Parameter:
//   devType    [IN]: 0=Eqoutip3; 1=Piccolo; 2=SilverSchmidt; 3=Profoscope; 10=Platform 
//   commPort   [IN]: 1=Serial(RS232); 2=USB; 4=Ethernet
//   scanPort   [IN]: true=scans serial ports upto COM255 or a device is found
//   portNr     [IN]: The COM port number the device is connected to.
//   ipAddress  [IN]: The IP address of the device e.g. "192.168.0.204"
//   waitMS     [IN]: Max time in [ms] to wait in this function. Note: If using 
//                    ethernet the underlying socket might still be working if 
//                    you set waitMS < 25000 ms.
// Return value:
//   0   :   No device found or timeout occurred.
//   >0  :   A handle to the device found. You need this for the other methods. 
PQREMOTE_API unsigned long CALLING_CONVENTION PQ_ConnectDevice(const unsigned short devType, 
                                            const unsigned short commPort,
                                            const bool           scanPort, 
                                            const unsigned short portNr, 
                                            const char          *ipAddress, 
                                            const unsigned long  waitMS);

//--- PQ_DisconnectDevice -----------------------------------------------------
// Description:
//   Disconnects the device assigned to deviceHandle. 
//   If the return value is false, the deviceHandle was not registered anymore.
PQREMOTE_API bool CALLING_CONVENTION PQ_DisconnectDevice(const unsigned long deviceHandle); 

//--- PQ_SendCMD --------------------------------------------------------------
// Description:
//   Send an ASCII command (cmd) to the device (deviceHandle) and receive the 
//   reply. The return value is the reply len or -1 in case of failure.
// Parameter:
//   deviceHandle [IN]: Handle to the device to send the cmd to.
//   cmd          [IN]: A null terminated ASCII command string including its 
//                      parameters.
//                      Samples: "@ID@"   "MATR 1"
//   reply        [IN]: A pointer to the buffer receiving the reply.
//               [OUT]: The reply from the device.
//   maxReplyLen  [IN]: The size of the buffer (reply) in bytes.
//   timeout      [IN]: The read/write timeout of the connection [ms]. 
//                      -1 = default timeout will be used.
// Return value:
//   -1  :  an error occurred (connection, timeout, parameter, wrong cmd)
//    0  :  The cmd was received by the device.
//   >0  :  The cmd was received and the device answered (reply). The return
//          value is the answer length in bytes.  
PQREMOTE_API long CALLING_CONVENTION PQ_SendCMD(const unsigned long deviceHandle, const char *cmd, 
                             char *reply, const unsigned long maxReplyLen,  
                             const long timeout);

//--- PQ_Read -----------------------------------------------------------------
// Description:
//   Read from the stream the device is connected to into the buffer.
//   the return value is the number of bytes read or -1 in case of failure.
// Parameter:
//   deviceHandle [IN]: Handle to the device to read from.
//   buffer       [IN]: A pointer to the buffer receiving the bytes read.
//               [OUT]: The bytes read from the device.
//   size         [IN]: The size of the buffer in bytes.
//   timeout      [IN]: The read timeout of the connection [ms]. 
//                      -1 = default timeout will be used.
// Return value:
//   -1  :  an error occurred (connection, timeout)
//    0  :  Wrong parameter.
//   >0  :  number of bytes read.  
PQREMOTE_API long CALLING_CONVENTION PQ_Read(const unsigned long deviceHandle, void *buffer, 
                         const unsigned long size, const long timeout); 



//=== typedefs, function pointers to the exported methods =====================

//--- Pointer to PQ_CheckCompatibility ----------------------------------------
typedef PQREMOTE_API unsigned long  (CALLING_CONVENTION * PQ_LPCHECKCOMPATIBILITY)(
                                           const unsigned short mainVersion, 
                                           const unsigned short sideVersion);  

//--- Pointer to PQ_ConnectDevice ---------------------------------------------
typedef PQREMOTE_API unsigned long  (CALLING_CONVENTION * PQ_LPCONNECTDEVICE)(
                                           const unsigned short devType, 
                                           const unsigned short commPort,
                                           const bool scanPort, 
                                           const unsigned short portNr, 
                                           const char *ipAddress, 
                                           const unsigned long waitMS);
  
//--- Pointer to PQ_DisconnectDevice ------------------------------------------
typedef PQREMOTE_API bool  (CALLING_CONVENTION * PQ_LPDISCONNECTDEVICE)(
                                           const unsigned long deviceHandle); 

//--- Pointer to PQ_SendCMD ---------------------------------------------------
typedef PQREMOTE_API long  (CALLING_CONVENTION * PQ_LPSENDCMD)(const unsigned long deviceHandle, 
                                           const char *cmd, char *reply, 
                                           const unsigned long maxReplyLen, 
                                           const long timeout);

//--- Pointer to PQ_Read ------------------------------------------------------
typedef PQREMOTE_API long  (CALLING_CONVENTION * PQ_LPREAD)(   const unsigned long deviceHandle, 
                                           void *buffer, 
                                           const unsigned long size, 
                                           const long timeout); 


//=== Some documentation ======================================================
// For a more detailed documentation, please consult PqRemote.pdf
//
// Proceq ASCII commands
// =====================
// Proceq ASCII commands are two (e.g. "ID") or four bytes (e.g. "MATR") 
// in length. Parameters (if any) follow after a space and are separated 
// with ';' (if more than one).
// With commands that allow requests, the request is a '?' (e.g. "MATR ?"
// 
// If the device could not process the command (unknown command or wrong
// parameter), it answers with a '?'. Else, the device processes the command
// and sends the answer:
// If Echo mode is off, the answer from the device is terminated with a 
// '\r' (0x0d). E.g. "MATR ?" --> "2\r"
// If the echo mode is on, the device echoes the received command followed 
// by a space and the answer, terminated with a "\r\n" (0x0d 0x0a). E.g.
// "MATR ?" --> "MATR ? 2\r\n"
//
// To enable echo mode, send "EC on" or "EC 1". 
// To disable it, send "EC off" or "EC 0".
// To request the current state, send "EC ?"
//
//
// NOTE (exceptions):
// - The HELP command answers ok with length 0. The effective answer must
// be read out afterwards using PQ_Read(). This is because the answer
// contains multiple "\r(\n)".
//
//
// A list of commands (use "HELP <command> to get more information):
//
// BR   switch baud rate,    EC   switch echo mode,    ID   get instrument ID,  
// @ID@ get ext.instrum.ID,  CSTD get/set conv. stand. DELF delete file/dir     
// DIRS get directory        DMSG disable message,     EMSG enable message      
// GETF load file to PC,     HELP show online help,    IAPV get instr.appl.vers.
// IHWR get instr. HW rev.,  IMPD set impact direction,INAM get instrument name,
// IOSV get instr. OS vers.  ISNO get instrument S/N,  KEYB keyboard I/O,       
// LOWR set lower limit,     MATR set material class,  MOVF move/rename file,   
// PAPV get probe appl.vers. PHWR get probe HW vers.,  PNAM get probe name,     
// POSV get probe OS version PSNO get probe S/N,       RLCK remote lock device, 
// RLGI remote login,        RLGO remote logout,       SCAL get/set conv. scale,
// SCSH screen shot,         SDEL delete reading,      SDEV get std. deviation, 
// SERM series close/reopen, SLEN get/set series len., SMAX get series max.val.,
// SMEA get series mean,     SMIN get series min.val., SNAM get series name,    
// SNOR get no. of readings, SRAN get series range,    SVAL get reading,        
// TIME get/set date & time, UPPR set upper limit                               

 


#ifdef __cplusplus
  }
#endif